(* -----------------------------------------------------------------------------
  Create Wallpaper     : 1.0
  Copyright  2014     : Copyright Adirondack Software & Graphics
  Last Modification    : 07-13-2014
  Source File          : Unit1.pas (Main Form)
  Compiler             : Delphi XE4 (Requires Delphi 2010 or higher)
  Target Platform      : Win32/Win64
  ImageEnVersion       : 5.1
  Operating System     : Windows 8.1
  Purpose              : ImageEnMView does not support transparency.  This
  : creates the illusion of transparency by creating a
  : bitmap from the image loaded into ImageEnView by using
  : a TRect whose coordinates correspond with TImageEnMView
  : ImageEnView.Bitmap as wallpaper.
  Lines of code        : 204
  Notes                :
  This file is copyright (C) W W Miller, 1986-2014.
  It may be used without restriction. This code distributed on an "AS IS"
  basis, WITHOUT WARRANTY OF ANY KIND, either express or implied.
  --------------------------------------------------------------------------- *)
unit Unit1;
{ Prevent platform compiler warnings }
{$WARN SYMBOL_PLATFORM OFF}
{$WARN UNIT_PLATFORM OFF}

interface

uses
  Winapi.Windows, Winapi.Messages, System.SysUtils, System.Variants,
  System.Classes, Vcl.Graphics, Vcl.Controls, Vcl.Forms, Vcl.Dialogs, iemview,
  ieview, imageenview, Vcl.ExtCtrls, Vcl.StdCtrls, Vcl.Menus, Vcl.ImgList,
  Vcl.ComCtrls;

type
  TForm1 = class(TForm)
    ImageEnView1: TImageEnView;
    Panel1: TPanel;
    ImageEnMView1: TImageEnMView;
    Fill1: TButton;
    Minimize1: TButton;
    Exit1: TButton;
    ImageList1: TImageList;
    LabelZoom1: TLabel;
    procedure ImageEnMView1AllDisplayed(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure ImageEnMView1ImageSelect(Sender: TObject; idx: Integer);
    procedure Fill1Click(Sender: TObject);
    procedure FormKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
    procedure Exit1Click(Sender: TObject);
    procedure Minimize1Click(Sender: TObject);
    procedure ImageEnView1ViewChange(Sender: TObject; Change: Integer);
  private
    procedure LoadWallpaper;
    { Private declarations }
  public
    { Public declarations }
  end;

var
  Form1: TForm1;

implementation

{$R *.dfm}

uses Winapi.ShlObj, Vcl.FileCtrl, iexHelperFunctions, hyieutils, hyiedefs,
  imageenproc;

function DesktopFolder: string;
{ Find Desktop folder location. }
var
  ir: bool;
  iPath: array [0 .. MAX_PATH] of char;
begin
  ir := Winapi.ShlObj.ShGetSpecialFolderPath(0, iPath, CSIDL_DESKTOP, False);
  if not ir then
    raise Exception.Create('Could not find Desktop folder location.');
  Result := IncludeTrailingPathDelimiter(iPath);
end;

function BrowseForFolder(AFolder: string): string;
{ Browse For Folder. }
begin
  if System.SysUtils.DirectoryExists(AFolder) then
    if Vcl.FileCtrl.SelectDirectory('Select Folder', 'Desktop', AFolder) then
      Result := AFolder;
end;

procedure TForm1.LoadWallpaper;
{ This was redesigned by Nigel Cross to improve performance
  with zoomed images. Note: The previous method copied a selection to a bitmap
  to create the wallpaper, but this method uses PainToEx which is faster and
  more accurate when the image is zoomed. }
var
  iBitmap: TBitmap;
  iIEBitmap: TIEBitmap;
  iLeft, iTop, iWidth, iHeight: Integer;
begin
  iLeft := Panel1.Left + ImageEnMView1.Left;
  iTop := Panel1.Top + ImageEnMView1.Top;
  iWidth := ImageEnMView1.Width;
  iHeight := ImageEnMView1.Height;
  iBitmap := TBitmap.Create;
  iBitmap.IEinitialize(iWidth, iHeight, ImageEnView1.Background);
  iIEBitmap := TIEBitmap.Create(ImageEnView1.ClientWidth,
    ImageEnView1.ClientHeight);
  try
    { Copy the full image to iIEBitmap }
    ImageEnView1.PaintToEx(iIEBitmap, nil, nil, true, true);
    { Copy a TRect of the IEBitmap to iBitmap }
    iBitmap.Canvas.CopyRect(Rect(0, 0, iWidth, iHeight), iIEBitmap.Canvas,
      Rect(iLeft, iTop, iLeft + iWidth, iTop + iHeight));
    { Make sure the panel is redrawn when zooming }
    Panel1.Repaint;
    { Use the iBitmap as wallpaper to simulate transparency }
    ImageEnMView1.WallPaper.Bitmap.Assign(iBitmap);
    { Refresh the wallpaper }
    ImageEnMView1.Update;
  finally
    iIEBitmap.Free;
    iBitmap.Free;
  end;
end;

procedure TForm1.Minimize1Click(Sender: TObject);
{ Minimize the window. }
begin
  Application.Minimize;
end;

procedure TForm1.Exit1Click(Sender: TObject);
{ Exit to windows. }
begin
  Close;
end;

procedure TForm1.Fill1Click(Sender: TObject);
{ Fill ImageEnMView with thumbnails from a folder. }
var
  iFolder: string;
begin
  iFolder := BrowseForFolder(DesktopFolder);
  if System.SysUtils.DirectoryExists(iFolder) then
  begin
    ImageEnMView1.Clear;
    ImageEnMView1.FillFromDirectory(iFolder);
    ImageEnMView1.SelectedImage := 0;
  end;
end;

procedure TForm1.FormCreate(Sender: TObject);
{ Set styling and add gesture support. }
begin
  ImageEnMView1.SetModernStyling();
  ImageEnView1.Gestures.Pan.Enabled := true;
  ImageEnView1.Gestures.Zoom.Enabled := true;
  ImageEnMView1.Gestures.Pan.Enabled := true;
  { Show the checkboxes only when a thumbnail is selected or the mouse hovers
    over the thumbnail }
  ImageEnMView1.Checkboxes := iecbShowOnHover;
  { Set the checkbox position to the topright }
  ImageEnMView1.CheckboxPos := iecpTopRight;
  ImageEnMView1.FillThumbnail := False;
  ImageEnMView1.ThumbnailsInternalBorder := False;
  ImageEnMView1.Background := clBlack;
  ImageEnMView1.DefaultTopTextFont.Color := clWhite;
  ImageEnMView1.DefaultBottomTextFont.Color := clWhite;
  ImageEnMView1.DefaultInfoTextFont.Color := clWhite;
end;

procedure TForm1.FormKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
{ Exit to windows when the ESC key is pressed. }
begin
  if Key = VK_ESCAPE then
    Close;
end;

procedure TForm1.ImageEnMView1AllDisplayed(Sender: TObject);
{ Load the full image from the thumbnail filename. }
begin
  if ImageEnMView1.SelectedImage = 0 then
  begin
    if ImageEnMView1.ImageCount > 0 then
    begin
      LabelZoom1.Caption := 'Zoom: ' + IntToStr(Round(ImageEnView1.Zoom)) + '%';
      { Load the image }
      ImageEnView1.IO.LoadFromFile(ImageEnMView1.MIO.Params
        [ImageEnMView1.SelectedImage].FileName);
      ImageEnView1.Stretch;
      LoadWallpaper;
    end;
  end;
end;

procedure TForm1.ImageEnMView1ImageSelect(Sender: TObject; idx: Integer);
{ ImageEnMView1ImageSelect. }
begin
  { Load the image }
  ImageEnView1.IO.LoadFromFile(ImageEnMView1.MIO.Params
    [idx].FileName);
  ImageEnView1.Stretch;
  LoadWallpaper;
end;

procedure TForm1.ImageEnView1ViewChange(Sender: TObject; Change: Integer);
begin
  LabelZoom1.Caption := 'Zoom: ' + IntToStr(Round(ImageEnView1.Zoom)) + '%';
  LoadWallpaper;
end;

end.
